Package: crlmm
Type: Package
Title: Genotype Calling (CRLMM) and Copy Number Analysis tool for
        Affymetrix SNP 5.0 and 6.0 and Illumina arrays
Version: 1.70.0
Author: Benilton S Carvalho, Robert Scharpf, Matt Ritchie, Ingo
        Ruczinski, Rafael A Irizarry
Maintainer: Benilton S Carvalho <benilton@unicamp.br>,
 Robert Scharpf <rscharpf@jhsph.edu>, Matt Ritchie
 <mritchie@wehi.EDU.AU>
Description: Faster implementation of CRLMM specific to SNP 5.0 and 6.0
        arrays, as well as a copy number tool specific to 5.0, 6.0, and
        Illumina platforms.
License: Artistic-2.0
Depends: R (>= 2.14.0), oligoClasses (>= 1.21.12), preprocessCore (>=
        1.17.7)
LinkingTo: preprocessCore (>= 1.17.7)
Imports: methods, Biobase (>= 2.15.4), BiocGenerics, affyio (>=
        1.23.2), illuminaio, ellipse, mvtnorm, splines, stats, utils,
        lattice, ff, foreach, RcppEigen (>= 0.3.1.2.1), matrixStats,
        VGAM, parallel, graphics, limma, beanplot
Suggests: hapmapsnp6, genomewidesnp6Crlmm (>= 1.0.7), snpStats, RUnit
Collate: AllGenerics.R AllClasses.R methods-AssayData.R methods-CNSet.R
        methods-CNSetLM.R methods-eSet.R methods-SnpSuperSet.R
        methods-PredictionRegion.R cnrma-functions.R cnset-accessors.R
        crlmm-functions.R crlmmGT2.R crlmm-illumina.R krlmm.R plot.R
        snprma-functions.R utils.R zzz.R test_crlmm_package.R
LazyLoad: yes
biocViews: Microarray, Preprocessing, SNP, CopyNumberVariation
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/crlmm
git_branch: RELEASE_3_23
git_last_commit: 416d6f5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:30:10 UTC; biocbuild
