##setGeneric("batch", function(object) standardGeneric("batch"))
##setGeneric("getParam", function(object, name, batch) standardGeneric("getParam"))
setGeneric("cnIndex", function(object) standardGeneric("cnIndex"))
setGeneric("cnNames", function(object) standardGeneric("cnNames"))
##setGeneric("computeCopynumber", function(object, ...) standardGeneric("computeCopynumber"))
##setGeneric("pr", function(object, name, batch, value) standardGeneric("pr"))
setGeneric("snpIndex", function(object) standardGeneric("snpIndex"))
##setGeneric("snpNames", function(object) standardGeneric("snpNames"))

setGeneric("CA", function(object, ...) standardGeneric("CA"))
setGeneric("CB", function(object, ...) standardGeneric("CB"))
setGeneric("totalCopynumber", function(object, ...) standardGeneric("totalCopynumber"))


setGeneric("Ns", function(object, ...) standardGeneric("Ns"))
setGeneric("corr", function(object, ...) standardGeneric("corr"))
setGeneric("mads", function(object, ...) standardGeneric("mads"))
setGeneric("medians", function(object, ...) standardGeneric("medians"))
setGeneric("tau2", function(object, ...) standardGeneric("tau2"))

## The generics below are for internal use with copy number methods
## If we keep them in oligoClasses, we need to export and document
setGeneric("N.AA", function(object) standardGeneric("N.AA"))
setGeneric("N.AB", function(object) standardGeneric("N.AB"))
setGeneric("N.BB", function(object) standardGeneric("N.BB"))
setGeneric("N.AA<-", function(object, value) standardGeneric("N.AA<-"))
setGeneric("N.AB<-", function(object, value) standardGeneric("N.AB<-"))
setGeneric("N.BB<-", function(object, value) standardGeneric("N.BB<-"))

setGeneric("medians", function(object, ...) standardGeneric("medians"))
setGeneric("medianA.AA", function(object) standardGeneric("medianA.AA"))
setGeneric("medianA.AB", function(object) standardGeneric("medianA.AB"))
setGeneric("medianA.BB", function(object) standardGeneric("medianA.BB"))
setGeneric("medianB.AA", function(object) standardGeneric("medianB.AA"))
setGeneric("medianB.AB", function(object) standardGeneric("medianB.AB"))
setGeneric("medianB.BB", function(object) standardGeneric("medianB.BB"))
setGeneric("medianA.AA<-", function(object, value) standardGeneric("medianA.AA<-"))
setGeneric("medianA.AB<-", function(object, value) standardGeneric("medianA.AB<-"))
setGeneric("medianA.BB<-", function(object, value) standardGeneric("medianA.BB<-"))
setGeneric("medianB.AA<-", function(object, value) standardGeneric("medianB.AA<-"))
setGeneric("medianB.AB<-", function(object, value) standardGeneric("medianB.AB<-"))
setGeneric("medianB.BB<-", function(object, value) standardGeneric("medianB.BB<-"))



setGeneric("madA.AA", function(object) standardGeneric("madA.AA"))
setGeneric("madA.AB", function(object) standardGeneric("madA.AB"))
setGeneric("madA.BB", function(object) standardGeneric("madA.BB"))
setGeneric("madB.AA", function(object) standardGeneric("madB.AA"))
setGeneric("madB.AB", function(object) standardGeneric("madB.AB"))
setGeneric("madB.BB", function(object) standardGeneric("madB.BB"))
setGeneric("madA.AA<-", function(object, value) standardGeneric("madA.AA<-"))
setGeneric("madA.AB<-", function(object, value) standardGeneric("madA.AB<-"))
setGeneric("madA.BB<-", function(object, value) standardGeneric("madA.BB<-"))
setGeneric("madB.AA<-", function(object, value) standardGeneric("madB.AA<-"))
setGeneric("madB.AB<-", function(object, value) standardGeneric("madB.AB<-"))
setGeneric("madB.BB<-", function(object, value) standardGeneric("madB.BB<-"))

setGeneric("tau2A.AA", function(object) standardGeneric("tau2A.AA"))
setGeneric("tau2A.BB", function(object) standardGeneric("tau2A.BB"))
setGeneric("tau2B.AA", function(object) standardGeneric("tau2B.AA"))
setGeneric("tau2B.BB", function(object) standardGeneric("tau2B.BB"))
setGeneric("tau2A.AA<-", function(object, value) standardGeneric("tau2A.AA<-"))
setGeneric("tau2A.BB<-", function(object, value) standardGeneric("tau2A.BB<-"))
setGeneric("tau2B.AA<-", function(object, value) standardGeneric("tau2B.AA<-"))
setGeneric("tau2B.BB<-", function(object, value) standardGeneric("tau2B.BB<-"))

setGeneric("corrAA", function(object) standardGeneric("corrAA"))
setGeneric("corrAB", function(object) standardGeneric("corrAB"))
setGeneric("corrBB", function(object) standardGeneric("corrBB"))
setGeneric("corrAA<-", function(object, value) standardGeneric("corrAA<-"))
setGeneric("corrAB<-", function(object, value) standardGeneric("corrAB<-"))
setGeneric("corrBB<-", function(object, value) standardGeneric("corrBB<-"))



setGeneric("nuA", function(object) standardGeneric("nuA"))
setGeneric("nuB", function(object) standardGeneric("nuB"))
setGeneric("phiA", function(object) standardGeneric("phiA"))
setGeneric("phiB", function(object) standardGeneric("phiB"))
setGeneric("phiPrimeA", function(object) standardGeneric("phiPrimeA"))
setGeneric("phiPrimeB", function(object) standardGeneric("phiPrimeB"))
setGeneric("phiPrimeA<-", function(object, value) standardGeneric("phiPrimeA<-"))
setGeneric("phiPrimeB<-", function(object, value) standardGeneric("phiPrimeB<-"))
setGeneric("tau2A", function(object) standardGeneric("tau2A"))
setGeneric("tau2B", function(object) standardGeneric("tau2B"))
setGeneric("nuA<-", function(object, value) standardGeneric("nuA<-"))
setGeneric("nuB<-", function(object, value) standardGeneric("nuB<-"))
setGeneric("phiA<-", function(object, value) standardGeneric("phiA<-"))
setGeneric("phiB<-", function(object, value) standardGeneric("phiB<-"))
setGeneric("tau2A<-", function(object, value) standardGeneric("tau2A<-"))
setGeneric("tau2B<-", function(object, value) standardGeneric("tau2B<-"))
setGeneric("flags<-", function(object, value) standardGeneric("flags<-"))

setGeneric("posteriorMean", function(object) standardGeneric("posteriorMean"))
setGeneric("posteriorMean<-", function(object, value) standardGeneric("posteriorMean<-"))

setGeneric("posteriorProbability", function(object, predictRegion, copyNumber=0:4, w)
	   standardGeneric("posteriorProbability"))
setGeneric("calculatePosteriorMean", function(object, posteriorProb, copyNumber=0:4, ...)
	   standardGeneric("calculatePosteriorMean"))

setGeneric("predictionRegion", function(object, copyNumber=0:4)
	   standardGeneric("predictionRegion"))
setGeneric("xyplot", useAsDefault=function(x, data, ...) lattice::xyplot(x, data,...))
setGeneric("xyplotcrlmm", function(x, data, predictRegion,...) standardGeneric("xyplotcrlmm"))
setGeneric("calculateRBaf", function(object, batch.name, chrom) standardGeneric("calculateRBaf"))

##setGeneric("mixtureParams<-", function(object,value) standardGeneric("mixtureParams<-"))
setGeneric("OligoSetList", function(object,...) standardGeneric("OligoSetList"))
setGeneric("BafLrrSetList", function(object,...) standardGeneric("BafLrrSetList"))
